import 'dart:io';
import 'package:path/path.dart' as path;

import 'api/api.dart';
import 'api/attribute.dart';
import 'api/tag.dart';

class HtmlSpecEntryFile {
  HtmlSpecEntryFile({
    required this.outputDirectory,
  });

  final String outputDirectory;

  StringBuffer? _stringBuffer;

  Future<void> write() async {
    final tags = await getHtmlTags();
    final globalAttributes = await getHtmlGlobalAttributes();

    for (final tag in tags) {
      _stringBuffer = StringBuffer();
      _writeHead();
      _writeImports();
      _writeTag(tag, globalAttributes);
      final outputFile =
          File(path.joinAll([outputDirectory, tag.name + '.g.dart']));
      outputFile.writeAsStringSync(_stringBuffer.toString());
    }

    final outputFile =
        File(path.joinAll([outputDirectory, 'html_elements.g.dart']));
    _stringBuffer = StringBuffer();
    _writeHead();
    for (final tag in tags) {
      _stringBuffer!.writeln("export '${tag.name}.g.dart';");
    }
    outputFile.writeAsStringSync(_stringBuffer.toString());
  }

  void _writeHead() {
    _stringBuffer!.writeln('// GENERATED CODE - DO NOT MODIFY BY HAND');
    _stringBuffer!.writeln('// GENERATED BY TODO');
    _stringBuffer!.writeln('// ignore_for_file: type=lint');
  }

  void _writeImports() {
    _stringBuffer!
        .writeln("import 'package:dart_html_elements/src/attribute.dart';");
    _stringBuffer!
        .writeln("import 'package:dart_html_elements/src/html_element.dart';");
  }

  void _writeTag(Tag tag, List<Attribute> globalAttributes) {
    final attributes = Set<Attribute>.from([
      ...globalAttributes,
      ...tag.attributes,
    ]);

    _stringBuffer!.writeAll(
      <String>[
        "/// ${tag.description}",
        "class ${tag.className} extends HTMLElement {",
        "    ${tag.className}({",
        for (final attribute in attributes) ...[
          if (attribute.description != null)
            "        /// ${attribute.description!.split('\n').join('\n        /// ')}",
          if (attribute.description == null)
            "        /// No documentation available.",
          "        final ${attribute.type}? ${attribute.argument},",
        ],
        "        final Map<String, String?>? dataAttributes,",
        "        final List<HTMLElement>? children,",
        "    })",
        " : super('${tag.name}', attributes: [",
        for (final attribute in attributes) ...[
          "        Attribute('${attribute.name}', ${attribute.argument}),",
        ],
        "        Attribute<Map<String, String?>?>('data', dataAttributes),",
        "], children: children,);",
        "}",
        "",
        "",
      ],
      '\n',
    );
  }
}
