import 'package:dart_html_elements/dart_html_elements.dart';

import 'attribute.dart';

String _renderAttributeAsString(Attribute attribute) {
  final name = attribute.name;
  final value = attribute.value;
  if (value == true) {
    return name;
  } else if (value == false) {
    return '';
  } else if (value is Function) {
    // TODO can't handle functions yet
    return '$name="console.error(\'Can\\\'t handle functions yet\');"';
  } else if (value is Map) {
    // e.g. data attributes, data-foo=1 data-bar=2
    return value.entries.map((entry) {
      if (entry.value == true) {
        return '$name-${entry.key}';
      } else if (entry.value == false) {
        return '';
      } else if (entry.value == null) {
        return '';
      }
      return '$name-${entry.key}="${entry.value}"';
    }).join(' ');
  } else if (value != null) {
    return '$name="$value"';
  } else {
    return '';
  }
}

String _attributesToString(List<Attribute>? attributes) {
  final attributesString = (attributes ?? [])
      .map((attribute) => _renderAttributeAsString(attribute))
      .where((string) => string.isNotEmpty)
      .join(' ');
  if (attributesString.isEmpty) {
    return '';
  }
  return ' $attributesString';
}

/// Renders the given [element] as an HTML string.
String renderAsString(HTMLElement? element) {
  if (element == null) {
    return '';
  }

  if (element is Text) {
    return element.text ?? '';
  }

  if (element is StatelessComponent) {
    return renderAsString(element.render());
  }

  final attributesString = _attributesToString(element.attributes);
  if (element.children != null) {
    final childrenString = element.children!.map(renderAsString).join('');
    return '<${element.tag}$attributesString>$childrenString</${element.tag}>';
  } else {
    return '<${element.tag}$attributesString/>';
  }
}

/// Stream renders the given [element] as an HTML string.
Stream<String> renderAsStream(HTMLElement? element) async* {
  if (element == null) {
    yield '';
    return;
  }

  if (element is Text) {
    yield element.text ?? '';
    return;
  }

  if (element is StatelessComponent) {
    yield* renderAsStream(element.render());
    return;
  }

  final attributesString = _attributesToString(element.attributes);
  if (element.children != null) {
    yield '<${element.tag}${attributesString}>';
    final children = Stream.fromIterable(element.children!);
    await for (final child in children) {
      yield* renderAsStream(child);
    }
    yield '</${element.tag}>';
  } else {
    yield '<${element.tag}${attributesString}/>';
  }
}
